#========================================================================================
# package      DE_EPAGES::Product::t::unit::ProductService_createProduct
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for DE_EPAGES::Product::API::ProductService
#               testing methods:
#               - createProduct()
#========================================================================================
package DE_EPAGES::Product::t::unit::ProductService_createProduct;
use base DE_EPAGES::Core::API::Test;

use DE_EPAGES::Core::API::Error qw ( Error );
use DE_EPAGES::Product::API::ProductService;
use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use DE_EPAGES::Test::Mock::Mockify qw ( GetParametersFromMockifyCall );
use DE_EPAGES::Test::Mock::Mockery;
use Test::Exception;
use Test::More;
use feature qw ( switch );
use JSON;

use strict;



sub test {
    my $self = shift;

    $self->test_createProduct_withoutProductNumber();
    $self->test_createProduct_withExistingProductNumber();
    $self->test_createProduct_withInvalidEAN();
    $self->test_createProduct_withInvalidUPC();
    $self->test_createProduct();

    done_testing();
}



sub test_createProduct_withoutProductNumber {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Mockery = $self->_createMocks();
    my $ProductService = $Mockery->get('SUT');
    my $Shop = $Mockery->get('Shop');

    my $hRequestBody = {};

    throws_ok(sub {
        $ProductService->createProduct($Shop, 'de_DE', 'EUR', $hRequestBody)
    },qr/BadRequest: Reason=The attribute productNumber is mandatory!/,
    "$SubTestName - test if Error BadRequest was thrown.");
}



sub test_createProduct_withExistingProductNumber {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Mockery = $self->_createMocks();
    my $ProductService = $Mockery->get('SUT');
    my $Shop = $Mockery->get('ShopWithExistingProduct');

    my $hRequestBody = {
        'productNumber' => 'product123',
    };

    throws_ok(sub {
        $ProductService->createProduct($Shop, 'de_DE', 'EUR', $hRequestBody)
    },qr/BadRequest: Reason=Product with given alias exists already!/,
    "$SubTestName - test if Error BadRequest was thrown.");
}



sub test_createProduct_withInvalidEAN {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Mockery = $self->_createMocks();
    my $ProductService = $Mockery->get('SUT');
    my $Shop = $Mockery->get('Shop');

    my $hRequestBody = {
        'productNumber' => 'product123',
        'ean' => 123,
    };

    throws_ok(sub {
        $ProductService->createProduct($Shop, 'de_DE', 'EUR', $hRequestBody)
    },qr/BadRequest: Reason=UPC or EAN is not valid!/,
    "$SubTestName - test if Error BadRequest was thrown.");
}



sub test_createProduct_withInvalidUPC {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Mockery = $self->_createMocks();
    my $ProductService = $Mockery->get('SUT');
    my $Shop = $Mockery->get('Shop');

    my $hRequestBody = {
        'productNumber' => 'product123',
        'upc' => 123,
    };

    throws_ok(sub {
        $ProductService->createProduct($Shop, 'de_DE', 'EUR', $hRequestBody)
    },qr/BadRequest: Reason=UPC or EAN is not valid!/,
    "$SubTestName - test if Error BadRequest was thrown.");
}



sub test_createProduct {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Mockery = $self->_createMocks();
    my $ProductService = $Mockery->get('SUT');
    my $Shop = $Mockery->get('Shop');

    my $hRequestBody = {
        'productNumber' => 'product123',
        'name' => 'test product',
        'description' => 'this is a test product',
        'shortDescription' => 'a test product',
        'manufacturer' => 'TestProductCompany',
        'essentialFeatures' => 'product for testing',
        'energyLabelsString' => 'test label string',
        'upc' => '4003994155486',
        'price' => 1.23,
        'isVisible' => JSON::true,
        'searchKeywords' => [
            'test',
            'product',
            'keywords'
        ]
    };

    my $hProductData = $ProductService->createProduct($Shop, 'de_DE', 'EUR', $hRequestBody);

    is_deeply($hProductData, $self->_createExpectedProductData, "$SubTestName - test if expected product data was returned.");

    my $aInsertObjectArgs = GetParametersFromMockifyCall($Mockery->get('ProductClass'), 'insertObject');
    is_deeply($aInsertObjectArgs->[0], {
        'Alias' => 'product123',
        'Parent' => 'testFolder',
    }, "$SubTestName - test if product was created with correct mandatory values.");

    my $Product = $Mockery->get('Product');
    is($Product->get('Name'), 'test product', "$SubTestName - test if name was set correctly.");
    is($Product->get('Text'), 'this is a test product', "$SubTestName - test if description was set correctly.");
    is($Product->get('Description'), 'a test product', "$SubTestName - test if shortDescription was set correctly.");
    is($Product->get('Manufacturer'), 'TestProductCompany', "$SubTestName - test if manufacturer was set correctly.");
    is($Product->get('MainCharacteristics'), 'product for testing', "$SubTestName - test if essentialFeatures was set correctly.");
    is($Product->get('EnergyLabelsString'), 'test label string', "$SubTestName - test if energyLabelsString was set correctly.");
    is($Product->get('UPCEAN'), '4003994155486', "$SubTestName - test if upc was set correctly.");
    is($Product->get('Keywords'), 'test, product, keywords', "$SubTestName - test if searchKeywords was set correctly.");
    ok($Product->get('IsVisible'), "$SubTestName - test if product is visible.");
    my $aSetListPriceArgs = GetParametersFromMockifyCall($Product, 'setListPrice');
    is($aSetListPriceArgs->[2], 1.23, "$SubTestName - test if listPrice was set correctly.");
}



sub _createMocks {
    my $self = shift;

    my $Mockery = DE_EPAGES::Test::Mock::Mockery->new();

    $Mockery->register('SUT', ['Locale'], sub {
        my ($Locale) = @_;
        my $Mockify = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::Product::API::ProductService', []);
        $Mockify->addMockWithReturnValueAndParameterCheck('_LoadClassByAlias', 'TestProductFolderClass', [{'string' => 'ProductFolder'}]);
        $Mockify->addMockWithReturnValueAndParameterCheck('_loadLocale', $Locale, ['string']);
        $Mockify->addMockWithReturnValueAndParameterCheck('getProductData', $self->_createExpectedProductData(), ['object', 'object', 'string', 'string']);
        $Mockify->mock('_Transaction', sub {
            my $MockedSelf = shift;
            my ($FktPtr) = @_;
            $FktPtr->();
        });
        return $Mockify->getMockObject();
    });
    $Mockery->register('Locale', [], sub {
        my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Object::API::Object::Locale', {
            'LanguageID' => '1',
        });
        return $Mockify->getMockObject();
    });
    my $ShopCreator = sub {
        my ($ProductClass) = @_;
        my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Shop::API::Object::Shop', {
            'Alias' => 'TestShop',
            'ProductClass' => $ProductClass,
            'TaxModel' => 1,
        });
        $Mockify->addMockWithReturnValueAndParameterCheck('folder', 'testFolder', ['string', 'hashref']);
        return $Mockify->getMockObject();
    };
    $Mockery->register('Shop', ['ProductClass'], $ShopCreator);
    $Mockery->register('ShopWithExistingProduct', ['ProductClassWithExistingProduct'], $ShopCreator);
    $Mockery->register('Product', [], sub {
        my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Product::API::Object::Product', {
            'Alias' => '',
            'Name' => '',
            'Text' => '',
            'Description' => ''
        });
        $Mockify->addMockWithReturnValueAndParameterCheck('folder', 'testFolder', ['string', 'hashref']);
        $Mockify->addMockWithReturnValueAndParameterCheck('setListPrice', undef, ['string', 'int', 'any']);
        return $Mockify->getMockObject();
    });
    $Mockery->register('ProductClass', ['Product'], sub {
        my ($Product) = @_;
        my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Object::API::Object::Class', {});
        $Mockify->addMockWithReturnValueAndParameterCheck('insertObject', $Product, ['hashref']);
        return $Mockify->getMockObject();
    });
    $Mockery->register('ProductClassWithExistingProduct', ['Product'], sub {
        my ($Product) = @_;
        my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Object::API::Object::Class', {});
        $Mockify->mock('insertObject', sub {
            Error('OBJECT_EXISTS');
        });
        return $Mockify->getMockObject();
    });

    return $Mockery->assemble();
}



sub _createExpectedProductData {
    return {
        'productNumber' => '123',
        'somedata' => 'test test',
    };
}

__PACKAGE__->runScript();